<?php
/**
 * Handler File Class
 *
 * @author liliang <liliang@wolive.cc>
 * @email liliang@wolive.cc
 * @date 2017/06/01
 */

namespace app\api\controller;

use think\Controller;
use think\Db;

/**
 * 基础验证
 */
class Base extends Controller
{
    /**
     * api接口基础验证，验证api请求合法性
     * @return array
     */
    public function _initialize()
    {
        if (empty($_GET['pop_token']) && empty($_GET['timestamp'])) {
            return $this->json(1, '缺少参数');
        }
        $timestamp = (int)$_GET['timestamp'];
        // 判断时间戳是否在5分钟以内
        $time_diff = time() - $timestamp;
        if ($time_diff > 300 || $timestamp < -300) {
            return $this->json(3, '非法请求.');
        }
        // 这些类不检查login_uid字段
        $exclude = ['app\api\controller\User', 'app\api\controller\Qrcode'];
        if (!in_array(get_class($this), $exclude)) {
            // 所有请求必须有login_uid
            if (!isset($_GET['login_uid']) && !isset($_POST['login_uid'])) {
                return $this->json(1, '缺少参数，login_uid');
            }
        }
        if ($_GET['pop_token'] !== md5($timestamp . \think\Config::get('pop.api_secret'))) {
            return $this->json(3, '非法请求..');
        }
    }

    /**
     * 合成json数据
     *
     * @param $code
     * @param string $msg
     * @param array $data
     * @return array
     */
    protected function json($code, $msg = '', $data = [])
    {
        $return = [
            'code' => $code,
            'msg'  => $msg,
            'data' => $data
        ];
        if (isset($_GET['internal_call'])) {
            return $return;
        }
        header('Content-Type: application/json;charset=utf-8');
        echo json_encode($return);
        die;
    }

    /**
     * 合成json数据
     *
     * @param $data
     * @return array
     */
    protected function jsonArray($data)
    {
        header('Content-Type: application/json;charset=utf-8');
        echo json_encode($data);
        die;
    }

    /**
     * 用户密码md5
     *
     * @param $username
     * @param $password
     * @return string
     */
    protected function md5($username, $password)
    {
        return md5("$username-popim-$password");
    }

    /**
     * 判断用户是否被禁用
     *
     * @param $uid
     * @return bool
     */
    protected function userDisabled($uid)
    {
        $account_state = Db::table('user')->where('uid', $uid)->value('account_state');
        return !$account_state || $account_state === 'disabled';
    }

    /**
     * 过滤$_GET
     * @return mixed
     */
    protected function _get()
    {
        array_walk_recursive($_GET, function (&$item){
            $item = htmlspecialchars($item);
        });
        return $_GET;
    }

    /**
     * 过滤$_POST
     * @return mixed
     */
    protected function _post()
    {
        array_walk_recursive($_POST, function (&$item){
            $item = htmlspecialchars($item);
        });
        return $_POST;
    }

    /**
     * 获取真实ip
     * @return string
     */
    public function getRealIp()
    {
        $remote_ip = $_SERVER['REMOTE_ADDR'];
        if (!$this->isIntranetIp($remote_ip)) {
            return $remote_ip;
        }
        if (isset($_SERVER["HTTP_X_FORWARDED_FOR"])) {
            $remote_ip = $_SERVER["HTTP_X_FORWARDED_FOR"];
        } elseif (isset($_SERVER["HTTP_CLIENT_IP"])) {
            $remote_ip = $_SERVER["HTTP_CLIENT_IP"];
        } elseif (isset($_SERVER["HTTP_X_REAL_IP"])) {
            $remote_ip = $_SERVER["HTTP_X_REAL_IP"];
        } elseif (isset($_SERVER["HTTP_X_CLIENT_IP"])) {
            $remote_ip = $_SERVER["HTTP_X_CLIENT_IP"];
        }
        return $remote_ip;
    }

    /**
     * 是否是内网ip
     * @param $ip
     * @return bool
     */
    protected function isIntranetIp($ip)
    {
        $reserved_ips = [
            '167772160'  => 184549375,  /*    10.0.0.0 -  10.255.255.255 */
            '3232235520' => 3232301055, /* 192.168.0.0 - 192.168.255.255 */
            '2130706432' => 2147483647, /*   127.0.0.0 - 127.255.255.255 */
            '2886729728' => 2887778303, /*  172.16.0.0 -  172.31.255.255 */
        ];

        $ip_long = ip2long($ip);

        foreach ($reserved_ips as $ip_start => $ip_end) {
            if (($ip_long >= $ip_start) && ($ip_long <= $ip_end)) {
                return true;
            }
        }
        return false;
    }


}
