<?php
namespace app\api\model;

use think\Model;
use think\Db;

/**
 * 个推离线推送相关服务
 */
class Getui extends Model
{

    /**
     * 推送服务构造函数
     *
     */
    public function __construct()
    {
        parent::__construct();
        require_once(__DIR__ . '/../../../vendor/getuilaboratory/getui-pushapi-php-client-v2/GTClient.php');
        if (!defined('GETUI_APPID')) {
            $setting = Db::table('setting')->field('getui_appid,getui_appkey,getui_master_secret, getui_api_address')->find();
            define('GETUI_APPID', $setting['getui_appid']);
            define('GETUI_APPKEY', $setting['getui_appkey']);
            define('GETUI_MASTER_SECRET', $setting['getui_master_secret']);
            define('GETUI_API_ADDRESS', $setting['getui_api_address']);
        }
    }


    public function getParam($title, $content) {
        $push = new \GTPushRequest();
        $push->setRequestId(time());
        //设置setting
        $set = new \GTSettings();
        $set->setTtl(3600*1000*23); // 23小时
        $strategy = new \GTStrategy();
        $strategy->setDefault(\GTStrategy::STRATEGY_THIRD_FIRST);
        $set->setStrategy($strategy);
        $push->setSettings($set);
        //设置PushMessage，
        $message = new \GTPushMessage();
        //通知
        $notify = new \GTNotification();
        $notify->setTitle($title);
        $notify->setBody($content);
        $notify->setClickType("none");
        $message->setNotification($notify);
        $push->setPushMessage($message);
        //厂商推送消息参数
        $pushChannel = new \GTPushChannel();
        //ios
        $ios = new \GTIos();
        $ios->setType("notify");
        $ios->setAutoBadge("1");
        //aps设置
        $aps = new \GTAps();
        $aps->setContentAvailable(0);

        $alert = new \GTAlert();
        $alert->setTitle($title);
        $alert->setBody($content);
        $aps->setAlert($alert);
        $ios->setAps($aps);
        $pushChannel->setIos($ios);
        //安卓
        $android = new \GTAndroid();
        $ups = new \GTUps();
        $thirdNotification = new \GTThirdNotification();
        $thirdNotification->setTitle($title);
        $thirdNotification->setBody($content);
        $thirdNotification->setClickType(\GTThirdNotification::CLICK_TYPE_STAERAPP);
        //$ups->addOption("HW","/message/android/notification/badge/add_num",1);
        //$ups->addOption("OP","channel","Default");
        //$ups->addOption("OP","aaa","bbb");
        //$ups->addOption(null,"a","b");

        $ups->setNotification($thirdNotification);
        $android->setUps($ups);
        $pushChannel->setAndroid($android);
        $push->setPushChannel($pushChannel);

        return $push;
    }

    public function push($cid_list, $title, $content, $payload=[])
    {
        if (!$cid_list) {
            return "cid list empty\n";
        }
        if (!GETUI_APPKEY) {
            return null;
        }
        $cid_list = is_array($cid_list) ? $cid_list : [$cid_list];

        $batch = new \GTPushBatchRequest();
        $pushes = [];
        foreach ($cid_list as $cid) {
            $push = $this->getParam($title, $content);
            $push->setCid($cid);
            $pushes[] = $push;
        }
        $batch->setMsgList($pushes);
        $async = config('unipush_async');
        $async = $async === null ? true : $async;
        $batch->setIsAsync($async);
        //$batch->setIsAsync(false);
        $api = new \GTClient(GETUI_API_ADDRESS,GETUI_APPKEY,GETUI_APPID,GETUI_MASTER_SECRET);

        try {
            return $api->pushApi()->pushBatchByCid($batch);
        } catch(\Throwable $e) {
            return (string)$e;
        }
    }

}
