<?php

namespace app\h5\controller;

use think\captcha\Captcha;
use think\Db;

class Login extends Base
{
    /**
     * 用户注册
     *
     * @return string
     */
    public function join()
    {
        if (!is_file(APP_PATH . '../config/database.php')) {
            return $this->json(-2, '服务端未安装');
        }

        $setting = Db::table('setting')->find();
        if ($setting['register'] == 'off') {
            return $this->json(1, '站点已经关闭注册');
        }

        if ($setting['register_captcha'] == 'on' && empty($_POST['captcha'])) {
            return $this->json(1, '缺少参数验证码');
        }
        // 默认头像
        $_POST['avatar'] = '/static/avatar.jpg';
        // 检查验证码
        $captcha = new Captcha();
        if ($setting['register_captcha'] == 'on' && !$captcha->check($_POST['captcha'], 'join')) {
            return $this->json(1, '验证码不正确');
        }
        $user = new \app\api\controller\User();
        $ret = $user->add();
        if ($ret['code'] != 0 || (empty($setting['default_friends']) && empty($setting['default_groups']))) {
            return $this->json($ret['code'], $ret['msg']);
        }
        $login_uid = $ret['data']['uid'];
        // 添加默认好友
        if (!empty($setting['default_friends'])) {
            $friend = new \app\api\controller\Friend();
            // 添加默认好友
            $uid_array = explode(',', $setting['default_friends']);
            foreach ($uid_array as $uid) {
                $_POST['login_uid'] = $login_uid;
                $_POST['friend_uid'] = $uid;
                $friend->add();
            }
        }

        // 添加默认群组
        if (!empty($setting['default_groups'])) {
            $group = new \app\api\controller\Group();
            // 添加默认好友
            $gid_array = explode(',', $setting['default_groups']);
            foreach ($gid_array as $gid) {
                $_POST['login_uid'] = $login_uid;
                $_POST['gid']       = $gid;
                $group->join();
            }
        }

        return $this->json($ret['code'], $ret['msg']);
    }

    /**
     * 登录验证
     */
    public function check()
    {
        if (!is_file(APP_PATH . '../config/database.php')) {
            return $this->json(-2, '服务端未安装');
        }

        if (empty($_POST['username']) || empty($_POST['password'])) {
            return $this->json(1, '缺少参数');
        }
        $_GET['username'] = $_POST['username'];
        $_GET['password'] = $_POST['password'];

        $setting = Db::table('setting')->find();
        if ($setting['login_captcha'] == 'on' && empty($_POST['captcha'])) {
            return $this->json(1, '缺少参数验证码');
        }
        // 检查验证码
        $captcha = new Captcha();
        if ($setting['login_captcha'] == 'on' && !$captcha->check($_POST['captcha'], 'login')) {
            return $this->json(1, '验证码不正确');
        }

        // 读取用户信息
        $user = new \app\api\controller\User();
        $user_info = $user->get();
        if ($user_info['code'] == 0) {
            if (session_status() !== PHP_SESSION_ACTIVE) {
                session_start();
            }
            $_SESSION['userinfo'] = $user_info['data'];
            if (!empty($_POST['cid'])) {
                Db::table('user')->where('cid', $_POST['cid'])->update(['cid' => '']);
                $_POST = [
                    'cid'       => $_POST['cid'],
                    'login_uid' => $user_info['data']['uid']
                ];
                $user->update();
            }
            return $this->json(0, 'ok', ['token' => session_id()]);
        } else {
            return $this->json($user_info['code'], $user_info['msg']);
        }
    }
    
    /**
     * 微信注册登录验证
     */
    public function wxcheck()
    {
        if (!is_file(APP_PATH . '../config/database.php')) {
            return $this->json(-2, '服务端未安装');
        }
    
        if (empty($_POST['code']) || empty($_POST['nickname'] || empty($_POST['avatar']))) {
            return $this->json(1, '缺少参数');
        }
        $appid = config('wx.appid');
        $secret = config('wx.appsecret');
        $code = $_POST['code'];
        $url = 'https://api.weixin.qq.com/sns/jscode2session?appid='.$appid.'&secret='.$secret.'&js_code=' . $code . '&grant_type=authorization_code';
        $result = file_get_contents($url);
        if (!$result) {
            return $this->json(500, '后台无法获取openid');
        }
        $data = json_decode($result, true);
        if (isset($data['errmsg'])) {
            return $this->json(501, $data['errmsg']);
        }
        if (!isset($data['openid'])) {
            return $this->json(502, '后台获取openid为空');
        }
        $openid = $data['openid'];
        $userinfo = Db::table('user')->where(['openid' => $openid])->find();
        if (!$userinfo) {
            $api = new \app\api\controller\Base();
            $userinfo = [
                'nickname'         => $_POST['nickname'],
                'avatar'           => $_POST['avatar'],
                'openid'           => $openid,
                'username'         => 'wx'.(Db::table('user')->max('uid')+1),
                'sign'             => '',
                'password'         => '',
                'logout_timestamp' => 0,
                'timestamp'        => time(),
                'first_char'       => (new \HanziToPinyin)->getInitials($_POST['nickname']),
                'register_ip'      => $api->getRealIp()
            ];
            
            Db::table('user')->insert($userinfo);
            $login_uid = Db::table('user')->getLastInsID();
            
            $setting = Db::table('setting')->find();
            
            // 添加默认好友
            if (!empty($setting['default_friends'])) {
                $friend = new \app\api\controller\Friend();
                // 添加默认好友
                $uid_array = explode(',', $setting['default_friends']);
                foreach ($uid_array as $uid) {
                    $_POST['login_uid'] = $login_uid;
                    $_POST['friend_uid'] = $uid;
                    $friend->add();
                }
            }
            
            // 添加默认群组
            if (!empty($setting['default_groups'])) {
                $group = new \app\api\controller\Group();
                // 添加默认好友
                $gid_array = explode(',', $setting['default_groups']);
                foreach ($gid_array as $gid) {
                    $_POST['login_uid'] = $login_uid;
                    $_POST['gid']       = $gid;
                    $group->join();
                }
            }
        } else {
            $login_uid = $userinfo['uid'];
        }
        if (session_status() !== PHP_SESSION_ACTIVE) {
            session_start();
        }
        $_SESSION['userinfo'] = [
            'uid'              => $login_uid,
            'nickname'         => $userinfo['nickname'],
            'avatar'           => $userinfo['avatar'],
            'username'         => $userinfo['username'],
        ];
        return $this->json(0);
    }

    /**
     * 注册验证码
     */
    public function joincaptcha()
    {
        $captcha = new Captcha(['length' => 4, 'fontSize' => 50]);
        return $captcha->entry('join');
    }

    /**
     * 登录验证码
     */
    public function logincaptcha()
    {
        $captcha = new Captcha(['length' => 4, 'fontSize' => 50]);
        return $captcha->entry('login');
    }

    /**
     * 退出
     */
    public function logout()
    {
        if (!isset($_POST['logout_uid'])) {
            return $this->json(1, '缺少参数');
        }
        if (session_status() !== PHP_SESSION_ACTIVE) {
            session_start();
        }
        if (empty($_SESSION['userinfo'])) {
            return $this->json(0, 'ok');
        }
        if ($_POST['logout_uid'] != $_SESSION['userinfo']['uid']) {
            return $this->json(2, '非法参数');
        }
        $_SESSION = null;
        session_destroy();
        if (!empty($_POST['cid'])) {
            Db::table('user')->where(['uid' => $_POST['logout_uid'], 'cid' => $_POST['cid']])->limit(1)->update(['cid' => '']);
        }

        return $this->json(0, 'ok');
    }

    /**
     * 获取配置
     */
    public function getsetting()
    {
        $pop = new \app\api\controller\Pop();
        $setting = $pop->getsetting();
        return $this->json($setting['code'], $setting['msg'], $setting['data']);
    }

    /**
     * 自动登录
     */
    public function autologin()
    {
        $salt = \think\Config::get('pop.api_secret');
        $request = $_REQUEST;
        $required = ['token', 'time', 'username', 'nickname', 'password'];
        $data = [];
        foreach ($required as $field) {
            if (!isset($request[$field])) {
                exit("$field missing");
            }
            $data[$field] = $request[$field];
        }
        unset($data['token'], $data['time']);

        $username = $request['username'];
        $token = $request['token'];
        $time = $request['time'];
        $timediff = abs(time() - $time);
        // 时间戳超过5分钟认为是过期的请求
        if ($timediff > 60*5) {
            exit('time error');
        }
        // 判断token是否正确
        if (md5($time.$username.$salt) !== $token) {
            exit("token error");
        }
        if ($userinfo = Db::table('user')->where('username', $username)->find()) {
            if (session_status() !== PHP_SESSION_ACTIVE) {
                session_start();
            }
            unset($data['password']);
            if (!isset($data['avatar'])) {
                $data['avatar'] = $userinfo['avatar'];
            }
            $data['uid'] = $userinfo['uid'];
            $_SESSION['userinfo'] = $data;
            header('Location: /im/h5');
            die;
        }

        if (!isset($data['avatar'])) {
            $data['avatar'] = '/static/avatar.jpg';
        }

        $_POST['username'] = $username;
        $_POST['nickname'] = $data['nickname'];
        $_POST['avatar'] = $data['avatar'];
        $_POST['password'] = $data['password'];
        $user = new \app\api\controller\User();
        $ret = $user->add();
        if ($ret['code'] != 0) {
            exit($ret['msg']);
        }
        $login_uid = $ret['data']['uid'];
        $setting = Db::table('setting')->find();
        // 添加默认好友
        if (!empty($setting['default_friends'])) {
            $friend = new \app\api\controller\Friend();
            $uid_array = explode(',', $setting['default_friends']);
            foreach ($uid_array as $uid) {
                $_POST['login_uid'] = $login_uid;
                $_POST['friend_uid'] = $uid;
                $friend->add();
            }
        }

        // 添加默认群组
        if (!empty($setting['default_groups'])) {
            $group = new \app\api\controller\Group();
            $gid_array = explode(',', $setting['default_groups']);
            foreach ($gid_array as $gid) {
                $_POST['login_uid'] = $login_uid;
                $_POST['gid']       = $gid;
                $group->join();
            }
        }

        if (session_status() !== PHP_SESSION_ACTIVE) {
            session_start();
        }
        $userinfo = Db::table('user')->where('username', $username)->find();
        unset($data['password']);
        $data['uid'] = $userinfo['uid'];
        $_SESSION['userinfo'] = $data;
        header('Location: /im/h5');
        die;
    }

    /**
     * 自动登录测试
     */
    /*function test() {
        $secret = \think\Config::get('pop.api_secret');
        $username = 'xiaoxiao2';
        $time = time();
        $nickname = '小小';
        $password = 'xiaoxiao';
        $avatar = '/static/avatar.jpg';
        $token = md5($time.$username.$secret);

        echo "<form method='post' action='/h5/login/autologin'>
<input type='hidden' name='token' value='$token'>
<input type='hidden' name='time' value='$time'>
<input type='hidden' name='username' value='$username'>
<input type='hidden' name='nickname' value='$nickname'>
<input type='hidden' name='password' value='$password'>
<input type='submit' name='打开im' value='打开im'>
</form>";
        die;
    }*/
}
